@@ -1,4 +1,6 @@
-# function from demo(error.catching) which silently catches a warning and includes it as a member of a returned list
+# ~ try() but catches warnings : 
+##' Catch *and* save both errors and warnings, and in the case of
+##' a warning, also keep the computed result.
 tryCatch.W.E <- function(expr) {
   W <- NULL
   w.handler <- function(w){ # warning handler
@@ -21,4 +23,12 @@ CBIND.Matrix <- function(x,y) cBind(x,y)
 
 RBIND <- function(x,y) UseMethod("RBIND") 
 RBIND.matrix <- function(x,y) rbind(x,as.matrix(y))
-RBIND.Matrix <- function(x,y) rBind(x,y)
\ No newline at end of file
+RBIND.Matrix <- function(x,y) rBind(x,y)
+
+overcat <- function(msg, prevmsglength) {
+  msglength <- nchar(msg)
+  if (prevmsglength>0) {cat("\r")}    
+  cat(msg)
+  return(msglength)
+}
+
